/**
 * Create an audio version of your posts, with a selection of more than 275+ voices across more than 48 languages and variants.
 * Exclusively on Envato Market: https://1.envato.market/speaker
 *
 * @encoding        UTF-8
 * @version         3.3.2
 * @copyright       Copyright (C) 2018 - 2021 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Alexander Khmelnitskiy (info@alexander.khmelnitskiy.ua), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

// Remove download button for web-kit players
window.addEventListener( 'DOMContentLoaded', (event) => {

    for ( let singleAudioTag of document.querySelectorAll( '.mdp-speaker-wrapper audio' ) ) {

        // Playback speed
        const $speeds = singleAudioTag.parentElement.parentElement.parentElement.querySelector( '.mdp-speaker--speed' );
        if ( $speeds !== null ){

            /** Init saved speed */
            const savedSpeed = getStorageSettings( 'speed' );

            /** Apply saved speed from the last visit */
            if ( savedSpeed ) {

                const $speedButtonSaved = document.querySelector( `button.mdp-speaker--speed-button[data-speed='${ savedSpeed }']` );

                if ( $speedButtonSaved ) {

                    speedClick( { target: $speedButtonSaved } );

                }

            }

            /** Listen for clicks */
            $speeds.addEventListener( 'click', speedClick );

            /**
             * Speed button click handler
             * @param e
             */
            function speedClick( e ) {

                const $speedButton = e.target;

                setSpeed( $speedButton.getAttribute( 'data-speed' ) );
                setSpeedButtonActive( $speeds, $speedButton );

            }

            /**
             * Set speed for Speaker player
             * @param speed
             */
            function setSpeed( speed ) {

                // WP Player
                if ( typeof window.MediaElementPlayer === 'function' ) {

                    new MediaElementPlayer( singleAudioTag.id );

                }

                singleAudioTag.playbackRate = speed;

                updateStorage( 'speed', speed );

            }

            /**
             * Set active-speed class for active button
             * @param $speeds
             * @param $speedButton
             */
            function setSpeedButtonActive( $speeds, $speedButton ) {

                $speeds.querySelectorAll( '.active-speed' ).forEach( element => element.classList.remove( 'active-speed' ) );
                $speedButton.classList.add( 'active-speed' );

            }

        }

        // Disallow downloading interface
        if ( singleAudioTag.parentElement.getAttribute( 'data-download' ) !== '1' ) {

            singleAudioTag.controlsList = 'nodownload';

        }

        // Chrome style player
        if ( singleAudioTag.parentElement.parentElement.classList.contains( 'speaker-chrome' ) ) {

            const $playerWrapper = singleAudioTag.parentElement.parentElement;
            let volumeShow = false;

            /** Show volume slider */
            $playerWrapper.addEventListener( 'mouseover', function ( e ) {

                if ( e.target.parentElement.classList.contains( 'mejs-volume-button' ) && ! volumeShow ) {

                    $playerWrapper.classList.remove( 'mejs-volume-hide' );
                    $playerWrapper.classList.add( 'mejs-volume-show' );
                    volumeShow = true;

                }

            }, { passive: true } );

            /** Hide volume slider */
            $playerWrapper.addEventListener( 'mouseout', function ( e ) {

                if ( e.target.className.length > 0 && ! e.target.className.includes( 'mejs-horizontal-volume' ) && volumeShow ) {

                    $playerWrapper.classList.add( 'mejs-volume-hide' );

                    setTimeout( () => {

                        $playerWrapper.classList.remove( 'mejs-volume-show' );
                        $playerWrapper.classList.remove( 'mejs-volume-hide' );

                    }, 100 );

                    volumeShow = false;

                }

            }, { passive: true } );

            /** Add the More menu button */
            setTimeout( () => {

                const $playerControls = $playerWrapper.querySelector( '.mejs-controls' );
                const $downloadLink = $playerWrapper.parentElement.querySelector( '.mdp-speaker-download-box a' );

                // Dont add more menu if download link is not exist
                if ( $downloadLink === null ) { return }

                // Add More menu button and menu
                const $moreButton = document.createElement( 'div' );
                $moreButton.classList.add( 'mejs-button' );
                $moreButton.classList.add( 'mejs-more-button' );
                $moreButton.innerHTML = '<button type="button" title="More" aria-label="More" tabindex="0"></button>';
                $moreButton.addEventListener( 'click', ( e ) => {

                    // Remove old menu or create new menu
                    if ( document.querySelector( '.mejs-more-menu' ) !== null ) {

                        document.querySelector( '.mejs-more-menu' ).remove();

                    } else {

                        // Add more menu
                        const $moreMenu = document.createElement( 'div' );
                        $moreMenu.classList.add( 'mejs-more-menu' );
                        $moreMenu.style.bottom = `${ e.layerY }px`;
                        $moreMenu.style.right = `${ e.layerX }px`;
                        $moreMenu.innerHTML = `<a href="${ $downloadLink.href }" class="mejs-more-menu--download" target="_blank" rel="noopener">Download</a>`;

                        $playerControls.appendChild( $moreMenu );

                    }

                }, { passive: false } )

                // Remove More menu if click somewhere
                document.addEventListener( 'click', ( e ) => {

                    // Exit if no one menu founded on the page
                    if ( document.querySelector( '.mejs-more-menu' ) === null ) { return; }

                    const elClassName = e.target.className;
                    if ( elClassName !== 'mejs-more-menu--download' && elClassName !== 'mejs-more-button' && elClassName !== '' ) {

                        document.querySelector( '.mejs-more-menu' ).remove();

                    }

                }, { passive: true } );

                $playerControls.appendChild( $moreButton );

            }, 1 );

        }

        /**
         * Get seetings from storage by key
         * @param key
         * @returns {boolean|*}
         */
        function getStorageSettings( key ) {

            let storedSettings = window.localStorage.getItem( 'mdpSpeaker' );

            if ( storedSettings ) {

                let localStorage = JSON.parse( storedSettings );
                return localStorage[ key ] ? localStorage[ key ] : false;

            } else {

                return false;

            }

        }

        /**
         * Update local storage
         * @param key
         * @param value
         */
        function updateStorage( key, value ) {

            const storage = window.localStorage.getItem( 'mdpSpeaker' );

            if ( storage ) {

                let localStorage = JSON.parse( storage );
                localStorage[ key ] = value;

                window.localStorage.setItem( 'mdpSpeaker', JSON.stringify( localStorage ) );

            } else {

                let localStorage = {};
                localStorage[ key ] = value;

                window.localStorage.setItem( 'mdpSpeaker', JSON.stringify( localStorage ) );

            }

        }

    }

});