<?php
/**
 * Create an audio version of your posts, with a selection of more than 275+ voices across more than 48 languages and variants.
 * Exclusively on Envato Market: https://1.envato.market/speaker
 *
 * @encoding        UTF-8
 * @version         3.3.2
 * @copyright       Copyright (C) 2018 - 2021 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Alexander Khmelnitskiy (info@alexander.khmelnitskiy.ua), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Speaker;

use Merkulove\Speaker;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;
}

/**
 * SINGLETON: Class adds admin styles.
 *
 * @since 3.0.0
 *
 **/
final class FrontScripts {

	/**
	 * The one true FrontScripts.
	 *
	 * @var FrontScripts
	 * @since 3.0.0
	 **/
	private static $instance;

	/**
	 * Sets up a new FrontScripts instance.
	 *
	 * @since 3.0.0
	 * @access public
	 **/
	private function __construct() {

		/** Add plugin scripts. */
		add_action( 'wp_enqueue_scripts', [ $this, 'enqueue_scripts' ] );

	}

	/**
	 * Add plugin scripts.
	 *
	 * @return void
	 * @since   1.0.0
	 **/
	public function enqueue_scripts() {

        if ( ! AssignmentsTab::get_instance()->display() ) { return; }

        /** Options shorthand */
        $options = Settings::get_instance()->options;

        /** Prepare conditions for script enqueue */
        $hide_download = 'speaker-browser-default' === $options[ 'style' ] && in_array( $options[ 'link' ], [ 'none', 'backend' ] );
        $enable_speeds = 'on' === $options[ 'speed_controls' ];
        $player_is_chrome = 'speaker-chrome' === $options[ 'style' ];

        /** Add JS script, for hide download button in the default webkit player */
        if ( $enable_speeds || $hide_download || $player_is_chrome ) {
            wp_enqueue_script( 'mdp-speaker', Speaker::$url . 'js/speaker' . Speaker::$suffix . '.js', [], Speaker::$version );
        }

        /** Remove WP mediaElement if set Default Browser Player. */
        if ( 'speaker-browser-default' === $options['style'] ) { return; }

        wp_enqueue_script( 'jquery' );

	}

	/**
	 * Main FrontScripts Instance.
	 *
	 * Insures that only one instance of FrontScripts exists in memory at any one time.
	 *
	 * @static
	 * @return FrontScripts
	 * @since 3.0.0
	 **/
	public static function get_instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof FrontScripts ) ) {

			self::$instance = new FrontScripts;

		}

		return self::$instance;

	}

} // End Class FrontScripts.
